#!/usr/bin/env node

const { spawn, spawnSync, exec } = require('child_process')
const options = { stdio: 'ignore', shell: true }
const fs = require('fs')

switch (process.argv[2]) {
  case 'auto-start':
    const s = '/etc/rc.local'
    if (!fs.existsSync(s)) {
      spawnSync(
        `
        touch ${s}
        echo '#!/bin/sh -e' >> ${s}
        echo 'mongo-rs start -o allow_other' >> ${s}
        chmod +x ${s}
      `,
        options
      )
    } else {
      const contents = fs
        .readFileSync(s, 'utf8')
        .replace('mongo-rs start -o allow_other', '')
        .trim()

      spawnSync(
        `
        rm ${s}
        touch ${s}
        echo '${contents}' >> ${s}
        echo 'mongo-rs start -o allow_other' >> ${s}
        chmod +x ${s}
      `,
        options
      )
    }
    break

  case 'kill':
    for (let index = 0; index < 3; index++) {
      let port = 27017 + index
      spawn(`kill $(sudo lsof -t -i:${port})`, {
        stdio: 'ignore',
        shell: true
      })
    }

    break

  case 'start':
    for (let index = 0; index < 3; index++) {
      let port = 27017 + index
      spawnSync(`mkdir -p /var/lib/mongodb/${port}`, options)
      spawn(
        `mongod --replSet rs --port ${port} --bind_ip 127.0.0.1 --dbpath /var/lib/mongodb/${port} --smallfiles --oplogSize 128`,
        {
          ...options,
          detached: true
        }
      )
    }

    let alreadyMessage = false
    const interval = setInterval(() => {
      exec(
        `
        mongo --port 27017 <<EOF
        rsconf = {_id: 'rs', members: [{_id: 0,host: '127.0.0.1:27017'},{_id: 1,host: '127.0.0.1:27018'},{_id: 2,host: '127.0.0.1:27019'}]}
        rs.initiate( rsconf )
        EOF`,
        (err, stdout, stderr) => {
          if (stdout.indexOf('AlreadyInitialized') > -1) {
            clearInterval(interval)
            if (!alreadyMessage) console.log('Successfully Initialized')
            alreadyMessage = true
          }
        }
      )
    }, 500)

  default:
    break
}
